(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/material')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/material'], factory) :
	(factory((global.ng = global.ng || {}, global.ng['mat-error-messages'] = global.ng['mat-error-messages'] || {}),global.ng.core,global.ng.material));
}(this, (function (exports,_angular_core,_angular_material) { 'use strict';

// Base class to enforce the passing of the fieldName to the constructor.
class ErrorMessages {
    constructor(fieldName) {
        this.fieldName = fieldName;
    }
}
// Token for injecting the default errors.
let DEFAULT_ERRORS = new _angular_core.OpaqueToken('DefaultErrors');
/** * Class of defaulted error messages to use with the MatErrorMessages directive.
 * fieldName will be passed in or defaulted.
 *
 * When a member is a method, it will be passed error object for that error.
 *
 * @export
 * @class DefaultErrorMessages
 */
class DefaultErrorMessages extends ErrorMessages {
    constructor() {
        super(...arguments);
        this.required = `${this.fieldName} is required`;
        this.numeric = `${this.fieldName} must be numeric.`;
        this.email = `Value is not a valid email address.`;
        this.pattern = `${this.fieldName} is in the wrong format`;
    }
    min(err) {
        return `${this.fieldName} must be more than ${err.min}.`;
    }
    max(err) {
        return `${this.fieldName} must be less than ${err.max}`;
    }
    minlength(err) {
        return `Max length is ${err.requiredLength} characters.`;
    }
    maxlength(err) {
        return `Max length is ${err.requiredLength} characters.`;
    }
}

/**
 * This 'directive' will display default app error messages in an <md-error> element
 * contained in an <md-form-field>.
 * @Usage:  <md-form-field>
 *                 <md-error matErrorMessages></md-error>
 *             </md-form-field>
 *
 * ** The input *must* be a FormControl and part of a FormGroup with Validators assigned to it.
 *
 * @Input clientMessages: An object whose keys are names of validators and values are error messages.
 *                             Including this will overwrite the default messag for a specific error.
 *
 * @Input fieldName: A string to use as the name of the field in the message. The MdInput placeholder
 *                      is used by default.
 *
 * @class ErrorMessageComponent
 * @implements {AfterViewInit}
 */
class MatErrorMessagesDirective {
    constructor(_inj) {
        this._inj = _inj;
        this.error = '';
        // This grabs a single active error instead of multiple.
        this.updateErrors = (state) => {
            if (state === 'INVALID') {
                const firstError = Object.keys(this.formErrors)[0];
                const messageGetter = this.errorList[firstError];
                this.error = Utils.callIfFunction(messageGetter, this.formErrors[firstError], this.inputRef.value);
            }
        };
    }
    get formErrors() {
        if (this.inputRef.ngControl) {
            return this.inputRef.ngControl.errors;
        }
    }
    get control() {
        return this.inputRef.ngControl;
    }
    // Setup all initial tooling
    ngAfterViewInit() {
        this._getInputRef();
        this.fieldName = this.fieldName || this.inputRef.placeholder;
        this.errorList = this._createErrorList();
        this._trySubToChanges();
    }
    // grab reference to MdInput directive, where form control is accessible.
    _getInputRef() {
        let container = this._inj.get(_angular_material.MdFormField);
        this.inputRef = container._control;
    }
    // Overwrite the default messages with supplied messages from the input.
    // We use a spread operator instead of Object.assign to retain getters and setters.
    _createErrorList() {
        const injectedErrors = this._inj.get(DEFAULT_ERRORS);
        const errorInstance = new injectedErrors(this.fieldName);
        // If no override messages given to directive, just return it as is.
        if (!this.clientMessages || typeof this.clientMessages !== 'object') {
            return errorInstance;
        }
        // If there are overrides, return new object with the defaults overridden.
        return Object.assign({}, errorInstance, this.clientMessages);
    }
    // Handles feedback on incorrect setup of the directive. If everything goes smoothly, sub to the control status!
    _trySubToChanges() {
        try {
            this._inj.get(_angular_material.MdError);
        }
        catch (err) {
            console.error('matErrorMessages: Directive must be on an <md-error>!');
        }
        if (!this.inputRef) {
            return console.error(`matErrorMEssages: Directive must be used in an <md-form-field>!`);
        }
        if (!this.inputRef.ngControl) {
            return console.error(`matErrorMessages: No FormControl registered for ${this.fieldName}!`);
        }
        
        this.control.statusChanges.subscribe(this.updateErrors);
    }
}
MatErrorMessagesDirective.decorators = [
    { type: _angular_core.Component, args: [{
                // tslint:disable-next-line:component-selector
                selector: '[matErrorMessages]',
                template: '{{ error }}'
            },] },
];
/** @nocollapse */
MatErrorMessagesDirective.ctorParameters = () => [
    { type: _angular_core.Injector, },
];
MatErrorMessagesDirective.propDecorators = {
    'clientMessages': [{ type: _angular_core.Input, args: ['matErrorMessages',] },],
    'fieldName': [{ type: _angular_core.Input, args: ['fieldName',] },],
};
class Utils {
    // Safe function checking against weird data types to see if something is a class.
    static isFunction(functionToCheck) {
        const getType = {};
        return functionToCheck && getType.toString.call(functionToCheck) === '[object Function]';
    }
    // Clean little utility...If the object passed is a function, it will return the function called with the provided args. If not, just returns the object.
    // oh, if it's a class it'll return a new instance of it.
    static callIfFunction(obj, args, ...rest) {
        const isFunction = Utils.isFunction(obj);
        return isFunction ? obj(args, ...rest) : obj;
    }
}

class MatErrorMessagesModule {
    static withDefaultErrors(errors) {
        return {
            ngModule: MatErrorMessagesModule,
            providers: [{
                    provide: DEFAULT_ERRORS,
                    useValue: errors
                }]
        };
    }
}
MatErrorMessagesModule.decorators = [
    { type: _angular_core.NgModule, args: [{
                declarations: [MatErrorMessagesDirective],
                exports: [MatErrorMessagesDirective],
                providers: [{
                        provide: DEFAULT_ERRORS,
                        useValue: DefaultErrorMessages
                    }]
            },] },
];
/** @nocollapse */
MatErrorMessagesModule.ctorParameters = () => [];

exports.ErrorMessages = ErrorMessages;
exports.DEFAULT_ERRORS = DEFAULT_ERRORS;
exports.DefaultErrorMessages = DefaultErrorMessages;
exports.MatErrorMessagesDirective = MatErrorMessagesDirective;
exports.MatErrorMessagesModule = MatErrorMessagesModule;

Object.defineProperty(exports, '__esModule', { value: true });

})));
