import { OpaqueToken } from '@angular/core';
// Base class to enforce the passing of the fieldName to the constructor.
export class ErrorMessages {
    constructor(fieldName) {
        this.fieldName = fieldName;
    }
}
// Token for injecting the default errors.
export let DEFAULT_ERRORS = new OpaqueToken('DefaultErrors');
/** * Class of defaulted error messages to use with the MatErrorMessages directive.
 * fieldName will be passed in or defaulted.
 *
 * When a member is a method, it will be passed error object for that error.
 *
 * @export
 * @class DefaultErrorMessages
 */
export class DefaultErrorMessages extends ErrorMessages {
    constructor() {
        super(...arguments);
        this.required = `${this.fieldName} is required`;
        this.numeric = `${this.fieldName} must be numeric.`;
        this.email = `Value is not a valid email address.`;
        this.pattern = `${this.fieldName} is in the wrong format`;
    }
    min(err) {
        return `${this.fieldName} must be more than ${err.min}.`;
    }
    max(err) {
        return `${this.fieldName} must be less than ${err.max}`;
    }
    minlength(err) {
        return `Max length is ${err.requiredLength} characters.`;
    }
    maxlength(err) {
        return `Max length is ${err.requiredLength} characters.`;
    }
}
//# sourceMappingURL=default-error-messages.js.map